/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv841x.h - Declaration file for utility functions and global variables
 *  DRV841x_Firmware
 *  05/25/2022
 *  Pablo Armet
 *
 ******************************************************************************/

#ifndef DRV841X_H_
#define DRV841X_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "msp430.h"
#include "dataTypeDefinition.h"
/******************************************************************************
 * MACROS
 *****************************************************************************/

/*
 * GPIO Port 1 Definitions
 */
#define STATUS      BIT0    // P1.0: MCU Status LED output
#define TXD         BIT1    // P1.1: UART TX
#define RXD         BIT2    // P1.2: UART RX
#define DRVID       BIT3    // P1.3: Device ID
#define VSENSE_B    BIT6    // P1.4: Sense output for H-bridge B
#define VSENSE_A    BIT7    // P1.5: Sense output for H-bridge A
#define P1_6        BIT6    // P1.6: Test point
#define P1_7        BIT7    // P1.7: Test point

/*
 * GPIO Port 2 Definitions
 */
#define P2_0        BIT0    // P2.0: Test Point
#define P2_1        BIT1    // P2.1: Test Point
#define P2_2        BIT2    // P2.2: Test Point
#define P2_3        BIT3    // P2.3: Test Point
#define nSLEEP      BIT4    // P2.4: DRV nSLEEP output
#define nFAULT      BIT5    // P2.5: DRV nFAULT output
#define PUSH        BIT6    // P2.6: PUSH button input
#define STALL       BIT7    // P2.7: STALL detection input

/*
 * GPIO Port 3 Definitions
 */
#define GUI_STATUS  BIT0    // P3.0: GUI Status LED output
#define P3_1        BIT1    // P3.1: Test point
#define BIN2        BIT2    // P3.2: BIN2 control output
#define BIN1        BIT3    // P3.3: BIN1 control output
#define P3_4        BIT4    // P3.4: Test Point
#define AIN2        BIT5    // P3.5: AIN2 control output
#define AIN1        BIT6    // P3.6: AIN1 control output
#define P3_7        BIT7    // P3.7: Test Point

/*SPI Protocol*/
/*#define SPI_BUSY_FLAG       0x01         // User define flag to check SPI busy status
#define SPI_ADDRESS_MASK    0x3F00       // Mask for SPI register address bit
#define SPI_ADDRESS_POS     8            // Position for SPI register address bit
#define SPI_DATA_MASK       0x00FF       // Mask for SPI register data bit
#define SPI_DATA_POS        0x0          // Position for SPI register data bit
#define SPI_RW_BIT_MASK     0x4000       // Mask for SPI register read write indication bit */

/*IO Macros*/
#define set_nSLEEP_hi          (P2OUT |= nSLEEP)        // Set nSLEEP high
#define set_nSLEEP_lo          (P2OUT &= ~nSLEEP)       // Set nSLEEP low

#define read_nFAULT_pin        (P2IN & nFAULT)          // Read FAULT_A status
#define read_nSLEEP_pin        (P2IN & nSLEEP)          // Read nSLEEP status
#define read_DRVID_pin         (P1IN & DRVID)           // Read Driver ID

#define MAX_PWM_DUTY        100
#define MIN_PWM_DUTY        0
#define _PWM_TIMER_PERIOD 100
#define PWM_TIMER_PERIOD (_PWM_TIMER_PERIOD-1)
#define PWM_SCALE (_PWM_TIMER_PERIOD/100)

#define stepper_acc            0                        // stepper motor acceleration variable

#define REV_8410               0                        // DRV8410EVM variant
#define REV_8411               1                        // DRV8411EVM variant

#define RESET_TIMEOUT       224     // cycle counts based on 16MHz clock and while loop execution

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/
/*
 * Ramp duty controller
 */
typedef struct DRV841x_RMPCNTL
{
  volatile uint8_t rampDelayMax;    // Parameter: Maximum delay rate (0-25)
  volatile uint8_t rampDelayCount;  // Variable: Incremental delay (0-25)
  volatile uint8_t setpointValue;   // Output: Target output(0-100)

} RMPCNTL_t;

typedef struct DRV841x_Device_Obj
{
  float firmwareVersion;

  /*Device Parameters*/
  volatile uint16_t deviceID;                             // 0: DRV8410; 1: DRV8411; 2: DRV8411A
  volatile uint8_t enable;                                // 0: Start Motor=0; 1: Start Motor=1
  volatile uint8_t DRV_FAULT;                             // 0: No Fault , 1: Fault
  volatile uint8_t ctr_mode;                              // 0: Parallel h-bridge, 1: Dual h-bridge
  volatile uint8_t step_rate;                             // 0: full step, 1: half-step
  volatile uint8_t dir;                                   // 0: forward , 1: reverse
  volatile uint16_t speed;                                // set motor speed when in stepper motor control mode
  volatile uint8_t ain1;                                  // set duty cycle for AIN1
  volatile uint8_t ain2;                                  // set duty cycle for AIN2
  volatile uint8_t bin1;                                  // set duty cycle for BIN1
  volatile uint8_t bin2;                                  // set duty cycle for BIN2
  volatile uint8_t motor_type;                            // 0: stepper motor, 1: Brushed DC motor
  volatile uint8_t freq;                                  // set PWM frequency
  volatile uint8_t nsleep;                                // Button; 0: nSLEEP=0 , 1: nSLEEP=1
  volatile uint8_t acc;                                   // acceleration rate in stepper motor
  volatile uint8_t decay_mode;                            // 0: slow decay, 1: fast decay

  /*Variables not in GUI*/
  volatile uint8_t bdc_acc;                               // acceleration rate in BDC motor
  volatile bool stepper_running;                          // state of stepper motor

  /*Ramp controller*/
  RMPCNTL_t rampInput1;                                   // AIN1
  RMPCNTL_t rampInput2;                                   // AIN2
  RMPCNTL_t rampInput3;                                   // BIN1
  RMPCNTL_t rampInput4;                                   // BIN2

} DRV841x_Device_Obj_t;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV841x_Device_Obj_t gBrushedObj;
//extern volatile DRV3946_Q1_REG_t gDeviceSpiReg;
//uint16_t spi_readRegister(uint8_t address);
//uint16_t spi_writeRegister(uint8_t address, uint16_t data);

/*Function Prototypes*/
void gui_deviceControlUpdateTask(void);
void firmwareInit(void);
void timer_reset(void);
void PWM_BDC_FrequencyControl(void);
void Stepper_Control(void);
void drv_rampControl(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue);
void StallDetectionControl(void);
void initMcu(void);
void getEVMID(void);
void drv841xevm_init(void);
void cmdRxIsr();
void uartTxByte(unsigned char byte);

#endif /* DRV841X_H_ */
