/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv841x.c - Definition file for utility functions
 *  DRV841xEVM_FIRMWARE
 *  05/25/2022
 *  Pablo Armet
 *
 *  v0.1: Initial Version
 *  v0.2: Updated Global variable data structure
 *  v0.3: Added functions for PWM selection and duty cycle
 *  v0.4: Fixed some changes to PWM_Freq_Sel_Dual(void) and PWM_Freq_Sel_Parallel(void) to fix innacurate duty cycle values and set duty cycle to 0 when < 0.5%
 *  v0.41: changed nFAULT pin (p2.5) from output to input
 *******************************************************************************/
#define FIRMWARE_VERSION 0.41

#include "drv841x.h"
#include "Serial_Cmd_Monitor.h"


/********************************************************************************
 * Local variable definition
 *******************************************************************************/
unsigned int temp_speed = 1000;                                     // variable for speed control in stepper motor
unsigned int temp_dir = 100;                                        //
unsigned int temp_ctr_mode = 100;                                   //
unsigned int temp_freq = 100;                                       //
unsigned int temp_enable = 100;                                     //
unsigned int temp_nsleep = 100;                                     //
unsigned int time_value = 1000;                                     //
unsigned int flag = 100;                                            //
unsigned int temp_motor_type = 100;                                 //
unsigned int temp_ain1 = 1000;
unsigned int temp_ain2 = 1000;
unsigned int temp_bin1 = 1000;
unsigned int temp_bin2 = 1000;
unsigned int temp_decay_mode = 1000;
unsigned int temp_step_rate = 1000;
//unsigned int PWM_SCALE = 0;

/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushedObj,".drvVarSection");
volatile DRV841x_Device_Obj_t gBrushedObj;

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/

/*
 * gui_deviceControlUpdateTask
 * Synchronize values in the global data structure written to by the GUI with
 * the hardware.
 */
void gui_deviceControlUpdateTask()
{
    if (gBrushedObj.nsleep > 0)
     set_nSLEEP_hi;                 // nSLEEP set HIGH
    else set_nSLEEP_lo;             // nSLEEP set LOW

    if(temp_motor_type != gBrushedObj.motor_type)
    {
        temp_motor_type = gBrushedObj.motor_type;
        if (gBrushedObj.motor_type == 0)                // stepper motor
        {
            timer_reset();                              // reset timer registers
            P3SEL &= ~(AIN1 | AIN2 | BIN1 | BIN2);      // set xINx as GPIO output
            TA1CCTL0 |= OUTMOD_7;                // Reset/Set
            TA1CTL |= MC_0 | TASSEL_2 | ID_0;    // stop timer, SMCLK, /1 division
        }
        else                                            // BDC motor
        {
            timer_reset();
            TA0CCTL1 |= OUTMOD_7;                       // Reset/Set
            TA0CCTL2 |= OUTMOD_7;                       // Reset/Set
            TA1CCTL1 |= OUTMOD_7;                       // Reset/Set
            TA1CCTL2 |= OUTMOD_7;                       // Reset/Set

            TA0CTL |= TASSEL_2 | MC_1;                  // SMCLK (2MHz), upmode
            TA1CTL |= TASSEL_2 | MC_1;                  // SMCLK (2MHz), upmode

            TA0CCR0 = PWM_TIMER_PERIOD;
            TA1CCR0 = PWM_TIMER_PERIOD;
        }
    }
} // end gui_deviceControlUpdateTask()

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
void getEVMID(void)
{
    /*
     * deviceID:
     * 0: DRV8410
     * 1: DRV8411
     * 2: DRV8411A
     */

    /* Identify the Device connected */
    uint8_t dev_id = 0;
    P2REN |= DRVID;                 // enable input resistor
    P2OUT |= DRVID;                 // input pull-up
    dev_id += read_DRVID_pin >> 1;   // store value
    P2OUT &= ~DRVID;                 // input pulldown
    dev_id += read_DRVID_pin >> 1;   // store value

    gBrushedObj.deviceID = dev_id;       // deviceID manual input
} // end getEVMID()

/*
 * MSP430G2553 Clock initialization
 */
static inline void clockInit(void)
{
    // 16 MHz DCO for MCLK
    DCOCTL  = 0x00;
    BCSCTL1 = CALBC1_16MHZ;
    DCOCTL  = CALDCO_16MHZ;

    // 2MHz SMCLK for PWM frequency and UART baud rate
    BCSCTL2 |= DIVS_3;
    BCSCTL3 |= LFXT1S_2;
} // end clockInit()

/*
 * UART initialization
 * 9600 BAUD
 */
static inline void uartInit(void)
{
    /* SMCLK */
    UCA0CTL1 |= UCSSEL_2;                      // SMCLK UART clock input

    /* 2MHz 9600 baud */
    UCA0BR0 = 0xd0;                            // Baud rate divider low byte
    UCA0BR1 = 0;                               // Baud rate divider high byte

    /* Modulation UCBRSx = 1 */
    UCA0MCTL = 0x92;                           // Baud rate divider modulation value

    /* Initialize USCI state machine */
    UCA0CTL1 &= ~UCSWRST;

    /* Enable USCI_A0 RX interrupt */
    IE2 |= UCA0RXIE;                           // UART RX interrupt enable

} // end uartInit()

/*
 * MSP430G2955 GPIO initialization
 */

static inline void gpioInit()
{
    // Port 1
    P1OUT = 0;
    P1DIR = 0;
    P1DIR  |= (STATUS);                         // Set outputs
    P1SEL  |= (RXD | TXD);                      // Enable UART signals
    P1SEL2 |= (RXD | TXD);
    ADC10AE0 |= (VSENSE_A | VSENSE_B);          // ADC inputs

    // Port 2
    P2OUT = 0;
    P2DIR = 0;
    P2DIR |= (nSLEEP | STALL);
    P2SEL = 0;
    P2SEL2 = 0;


    // Port 3
    P3OUT = 0;
    P3DIR = 0;
    P3DIR |= (GUI_STATUS | AIN1 | AIN2 | BIN1 | BIN2);
    P3SEL |= (AIN1 | AIN2 | BIN1 | BIN2);        // Timer outputs
    P3SEL2 = 0;

    /* Setup PUSH button for triggering IRQ */
    // External pull up resistor 47kOhms R18
    P1DIR &= ~PUSH;   // input mode
    P1IES |= PUSH;    // falling edge
    P1IFG &= ~PUSH;   // clear IRQ that may have been generated by setting IES
    // P1IE  |= PUSH;    // enable IRQ on PUSH button press **not enabled in this firmware version**
} // end gpioInit()

/*
 * Setup Watchdog Timer as an Interval Timer
 * Timer Interval to periodically wake up MSP430G2955 from LPM0
 */
static inline void watchdogTimerInit(void)
{
    WDTCTL = WDT_ADLY_1_9;                     // Configure time interval for periodic wake up and processing 1.9 ms if using 32 kHz ACLK, with VLO ~4 ms
    IE1 |= WDTIE;                              // Watchdog interval timer Interrupt enable
} // end watchdogTimerInit()

void initMcu(void)
{
    // Stop Watchdog Timer
    WDTCTL = WDTPW | WDTHOLD;

    // Setup CLKs, GPIO and Peripherals
    gpioInit();                                // MSP430G2553 GPIOI initialization
    clockInit();                               // MSP430G2553 basic clock initialization
    uartInit();                                // MSP430G2553 UART initialization
    watchdogTimerInit();

} // end initMcu()

void firmwareInit(void)
{
    /*EVM Firmware version configuration*/
    gBrushedObj.firmwareVersion = FIRMWARE_VERSION;

    /*Ramp controller init for AIN1*/
    gBrushedObj.rampInput1.rampDelayMax   = 25;
    gBrushedObj.rampInput1.rampDelayCount = 25;
    gBrushedObj.rampInput1.setpointValue  = 80;

    /*Ramp controller init  for AIN2*/
    gBrushedObj.rampInput2.rampDelayMax = 25;
    gBrushedObj.rampInput2.rampDelayCount = 10;
    gBrushedObj.rampInput2.setpointValue = 80;

    /*Ramp controller init for BIN1*/
    gBrushedObj.rampInput3.rampDelayMax   = 25;
    gBrushedObj.rampInput3.rampDelayCount = 10;
    gBrushedObj.rampInput3.setpointValue  = 80;

    /*Ramp controller init  for BIN2*/
    gBrushedObj.rampInput4.rampDelayMax = 25;
    gBrushedObj.rampInput4.rampDelayCount = 10;
    gBrushedObj.rampInput4.setpointValue = 80;

    /*Read device ID*/
    getEVMID();
}

// Resets timer registers
void timer_reset(void)
{
    TA0CTL = TA1CTL = 0;
    TA0CCTL0 = TA0CCTL1 = TA0CCTL2 = 0;
    TA1CCTL0 = TA1CCTL1 = TA1CCTL2 = 0;
    TA0CCR0 = TA1CCR0 = 0;
    TA0CCR1 = TA0CCR2 = 0;
    TA1CCR1 = TA1CCR2 = 0;
}

void drv_rampControl(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue)
{
    if (rmpCntl->rampDelayMax == 0)
    {
        rmpCntl->setpointValue = targetValue;
    }

    else
    {
        if (rmpCntl->rampDelayCount++ >= rmpCntl->rampDelayMax)
        {
            if (targetValue > rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue++;
            }
            else if(targetValue < rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue--;
            }
            rmpCntl->rampDelayCount = 0;
        }
    }
}
void PWM_BDC_FrequencyControl(void)  // Set PWM frequency and duty cycle for XIN1 and XIN2
{

// change PWM frequency when gBrushedObj.freq changes state.
    if (temp_freq != gBrushedObj.freq)
    {
        temp_freq = gBrushedObj.freq;
        // divide timer clock only when nSLEEP=0 and Brushed DC motor is chosen
        if (gBrushedObj.nsleep == 0 && gBrushedObj.motor_type==1)
        {
            switch(gBrushedObj.freq)
            {
            case 0:                             // 20kHz
                TA0CTL    &= ~ID_3;
                TA1CTL    &= ~ID_3;
                TA0CTL    |= ID_0 | TACLR;
                TA1CTL    |= ID_0 | TACLR;
                //PWM_SCALE = 1.07;              // offset scale
                break;
            case 1:                             // 10kHz
                TA0CTL    &= ~ID_3;
                TA1CTL    &= ~ID_3;
                TA0CTL    |= ID_1 | TACLR;
                TA1CTL    |= ID_1 | TACLR;
               // PWM_SCALE = 1;              // offset scale
                break;
            case 2:                             // 5kHz
                TA0CTL    &= ~ID_3;
                TA1CTL    &= ~ID_3;
                TA0CTL    |= ID_2 | TACLR;
                TA1CTL    |= ID_2 | TACLR;
               // PWM_SCALE = 0.50;              // offset scale
                break;
            case 3:                             // 2.5kHz
                TA0CTL    |= ID_3 | TACLR;
                TA1CTL    |= ID_3 | TACLR;
               // PWM_SCALE = 0.3;              // offset scale
                break;
            }
        }
    }

    if (temp_ain1 != gBrushedObj.ain1 || temp_ain2 != gBrushedObj.ain2 ||
        temp_bin1 != gBrushedObj.bin1 || temp_bin2 != gBrushedObj.bin2 ||
        temp_ctr_mode != gBrushedObj.ctr_mode)
    {
        temp_ain1 = gBrushedObj.ain1;
        temp_ain2 = gBrushedObj.ain2;
        temp_bin1 = gBrushedObj.bin1;
        temp_bin2 = gBrushedObj.bin2;
        temp_ctr_mode = gBrushedObj.ctr_mode;

        if (gBrushedObj.nsleep == 0)
        {
            P3SEL &= ~(AIN1|AIN2|BIN1|BIN2);
            P3OUT &= ~(AIN1|AIN2|BIN1|BIN2);
        }

        if(gBrushedObj.nsleep==1 && gBrushedObj.ctr_mode==0 && gBrushedObj.motor_type==1) //device enabled and parallel H-bridge and BDC motor
        {
            if (gBrushedObj.ain1 == MAX_PWM_DUTY)
            {
                TA0CTL &= ~TAIE;                                    // disable interrupts.
                TA0CCTL2 &= ~CCIE;
                P3SEL &= ~(AIN1 | BIN1);
                P3OUT |= (AIN1 | BIN1);
            }
            if (gBrushedObj.ain2 == MAX_PWM_DUTY)
            {
                TA0CTL &= ~TAIE;                                    // disable interrupts.
                TA0CCTL1 &= ~CCIE;
                P3SEL &= ~(AIN2 | BIN2);
                P3OUT |= (AIN2 | BIN2);
            }
            if (gBrushedObj.ain1 == MIN_PWM_DUTY)
            {
                TA0CTL &= ~TAIE;                                    // disable interrupts.
                TA0CCTL2 &= ~CCIE;
                P3SEL &= ~(AIN1 | BIN1);
                P3OUT &= ~(AIN1 | BIN1);
            }
            if (gBrushedObj.ain2 == MIN_PWM_DUTY)
            {
                TA0CTL &= ~TAIE;                                    // disable interrupts.
                TA0CCTL1 &= ~CCIE;
                P3SEL &= ~(AIN2 | BIN2);
                P3OUT &= ~(AIN2 | BIN2);
            }
            if (gBrushedObj.ain1 > MIN_PWM_DUTY && gBrushedObj.ain1 < MAX_PWM_DUTY)
            {
                P3SEL &= ~(AIN1 | BIN1);
                TA0CCTL2 |= CCIE;
                TA0CTL |= TAIE;
               // drv_rampControl(&gBrushedObj.rampInput1, gBrushedObj.ain1);
                TA0CCR2 = gBrushedObj.ain1;
            }
            if (gBrushedObj.ain2 > MIN_PWM_DUTY && gBrushedObj.ain2 < MAX_PWM_DUTY)
            {
                P3SEL &= ~(AIN2 | BIN2);
                TA0CCTL1 |= CCIE;
                TA0CTL |= TAIE;
               // drv_rampControl(&gBrushedObj.rampInput1, gBrushedObj.ain1);
                TA0CCR1 = gBrushedObj.ain2;

            }
        }

        if(gBrushedObj.nsleep==1 && gBrushedObj.ctr_mode==1 && gBrushedObj.motor_type==1) //device enabled and dual H-bridge and BDC motor
        {
            TA0CTL &= ~TAIE;                                    // disable interrupts.
            TA0CCTL1 &= ~CCIE;
            TA0CCTL2 &= ~CCIE;

            if (gBrushedObj.ain1 == MAX_PWM_DUTY)
            {
                P3SEL &= ~AIN1;
                P3OUT |= AIN1;
            }
            if (gBrushedObj.ain2 == MAX_PWM_DUTY)
            {
                P3SEL &= ~AIN2;
                P3OUT |= AIN2;
            }
            if (gBrushedObj.bin1 == MAX_PWM_DUTY)
            {
                P3SEL &= ~BIN1;
                P3OUT |= BIN1;
            }
            if (gBrushedObj.bin2 == MAX_PWM_DUTY)
            {
                P3SEL &= ~BIN2;
                P3OUT |= BIN2;
            }
            if (gBrushedObj.ain1 == MIN_PWM_DUTY)
            {
                P3SEL &= ~AIN1;
                P3OUT &= ~AIN1;
            }
            if (gBrushedObj.ain2 == MIN_PWM_DUTY)
            {
                P3SEL &= ~AIN2;
                P3OUT &= ~AIN2;
            }
            if (gBrushedObj.bin1 == MIN_PWM_DUTY)
            {
                P3SEL &= ~BIN1;
                P3OUT &= ~BIN1;
            }
            if (gBrushedObj.bin2 == MIN_PWM_DUTY)
            {
                P3SEL &= ~BIN2;
                P3OUT &= ~BIN2;
            }
            if (gBrushedObj.ain1 > MIN_PWM_DUTY && gBrushedObj.ain1 < MAX_PWM_DUTY)
            {
                P3SEL |= AIN1;
               // drv_rampControl(&gBrushedObj.rampInput1, gBrushedObj.ain1);
                TA0CCR2 = gBrushedObj.ain1;
            }
            if (gBrushedObj.ain2 > MIN_PWM_DUTY && gBrushedObj.ain2 < MAX_PWM_DUTY)
            {
                P3SEL |= AIN2;
               // drv_rampControl(&gBrushedObj.rampInput2, gBrushedObj.ain2);
                TA0CCR1 = gBrushedObj.ain2;
            }
            if (gBrushedObj.bin1 > MIN_PWM_DUTY && gBrushedObj.bin1 < MAX_PWM_DUTY)
            {
                P3SEL |= BIN1;
               // drv_rampControl(&gBrushedObj.rampInput3, gBrushedObj.bin1);
                TA1CCR2 = gBrushedObj.bin1;
            }
            if (gBrushedObj.bin2 > MIN_PWM_DUTY && gBrushedObj.bin2 < MAX_PWM_DUTY)
            {
                P3SEL |= BIN2;
              //  drv_rampControl(&gBrushedObj.rampInput4, gBrushedObj.bin2);
                TA1CCR1 = gBrushedObj.bin2;
            }
        }
    }

}  // end PWM_FrequencyControl()

void Stepper_Control(void)
{
    if(gBrushedObj.motor_type == 0)                 // Stepper motor
    {
        switch(gBrushedObj.step_rate)               // step rate
        {
        case 0:         // full-step
            if(temp_dir != gBrushedObj.dir || temp_speed != gBrushedObj.speed || temp_enable != gBrushedObj.enable)
            {
                temp_speed = gBrushedObj.speed;
                time_value = 2000000 / gBrushedObj.speed;
                temp_dir = gBrushedObj.dir;
                temp_enable = gBrushedObj.enable;
                //configure TA1.0 timer used for interrupts
                TA1CCR0 = time_value >> 2;                       // clock frequency 1/4 of time_value
                temp_enable = gBrushedObj.enable;
                if(gBrushedObj.enable == 1)
                {
                    //TA1CCTL1 |= CCIE;                       // Enable CCR1 interrupt
                    //TA1CCTL0 |= CCIE;                       // Enable CCR0 interrupt
                    //TA1CCTL2 |= CCIE;                       // Reset/Set
                    TA1CTL |= MC_1 |TAIE;                     // up count mode, interrupt enabled
                    gBrushedObj.stepper_running = true;
                }
                else
                {
                    TA1CTL &= ~ MC_3;               // stop timer and disable interrupt
                    P3OUT &= ~(AIN1|AIN2|BIN1|BIN2);
                    //TA1CCTL1 &= ~CCIE;                      // disable CCR1 interrupt
                    //TA1CCTL0 &= ~CCIE;                      // disable CCR0 interrupt
                    //TA1CCTL2 &= ~ CCIE;
                    gBrushedObj.stepper_running = false;
                }
            }
            break;
        case 1:         // half-step
            if(temp_dir != gBrushedObj.dir || temp_speed != gBrushedObj.speed || temp_enable != gBrushedObj.enable)
            {
                temp_speed = gBrushedObj.speed;
                time_value = 2000000 / gBrushedObj.speed;
                temp_dir = gBrushedObj.dir;
                temp_enable = gBrushedObj.enable;
                //configure TA1.0 timer used for interrupts
                TA1CCR0 = time_value >> 3;                       // clock frequency 1/8 of time_value
                temp_enable = gBrushedObj.enable;
                if(gBrushedObj.enable == 1)
                {
                    //TA1CCTL1 |= CCIE;                       // Enable CCR1 interrupt
                    //TA1CCTL0 |= CCIE;                       // Enable CCR0 interrupt
                    //TA1CCTL2 |= CCIE;                       // Reset/Set
                    TA1CTL |= MC_1 |TAIE;                  // up count mode, interrupt enabled
                    gBrushedObj.stepper_running = true;
                }
                else
                {
                    TA1CTL &= ~ MC_3;               // stop timer and disable interrupt
                    P3OUT &= ~(AIN1|AIN2|BIN1|BIN2);
                    //TA1CCTL1 &= ~CCIE;                      // disable CCR1 interrupt
                    //TA1CCTL0 &= ~CCIE;                      // disable CCR0 interrupt
                    //TA1CCTL2 &= ~ CCIE;
                    gBrushedObj.stepper_running = false;
                }
            }
            break;
        }
    }
} // end Stepper_Control()

void StallDetectionControl(void)
{
    //TBD
} // end Stepper_Control
void cmdRxIsr()
{
// disable UART interrupt
    IE2 &= ~UCA0RXIE;
    __no_operation();
    __no_operation();
    __bis_SR_register(GIE); // re-enabled global interrupts to allow timer interrupts during UART RX interrupt service routine
//
    /* USCI_A0 TX buffer ready? */
    while (!(IFG2 & UCA0TXIFG));

    /* Parse received command immediately */
    receivedDataCommand(UCA0RXBUF);

// re-enable UART interrupt
    __bic_SR_register(GIE); // disable global interrupt
    __no_operation();
    __no_operation();
    IE2 |= UCA0RXIE;
} // end cmdRxIsr()

void uartTxByte(unsigned char byte)
{
    /* USCI_A0 TX buffer ready? */
    while (!(IFG2 & UCA0TXIFG))
        ;
    /* transmit data */
    UCA0TXBUF = byte;
} // end uartTxByte()

